"""
Define some generators based on the caching mechanism that
are commonly needed for code generation
"""

from dune.perftool.generation import generator_factory
from dune.perftool.cgen.clazz import AccessModifier

symbol = generator_factory(item_tags=("symbol",))
preamble = generator_factory(item_tags=("preamble",), counted=True)


def pre_include(pre, filetag=None, pre_include=True):
    assert filetag
    gen = generator_factory(item_tags=("file", filetag, "pre_include"), no_deco=True)
    return gen(pre)


def post_include(post, filetag=None, pre_include=True):
    assert filetag
    gen = generator_factory(item_tags=("file", filetag, "post_include"), no_deco=True)
    return gen(post)


def include_file(include, filetag=None):
    assert filetag
    from cgen import Include
    gen = generator_factory(on_store=lambda i: Include(i), item_tags=("file", filetag, "include"), no_deco=True)
    return gen(include)


def initializer_list(obj, params, classtag=None):
    assert classtag
    gen = generator_factory(item_tags=("clazz", classtag, "initializer"), counted=True, no_deco=True, cache_key_generator=lambda *a: a[0])
    return gen("{}({})".format(obj, ", ".join(params)))


def base_class(baseclass, classtag=None, access=AccessModifier.PUBLIC, construction=[]):
    assert classtag

    from dune.perftool.cgen.clazz import BaseClass
    gen = generator_factory(item_tags=("clazz", "baseclass", classtag), on_store=lambda n: BaseClass(n, inheritance=access), counted=True, no_deco=True)

    if construction:
        initializer_list(baseclass, construction, classtag=classtag)

    return gen(baseclass)


def class_member(classtag=None, access=AccessModifier.PRIVATE):
    assert classtag
    from cgen import Value
    from dune.perftool.cgen.clazz import ClassMember

    return generator_factory(item_tags=("clazz", classtag, "member"), on_store=lambda m: ClassMember(m, access=access), counted=True)


def constructor_parameter(_type, name, classtag=None, constructortag="default"):
    assert classtag
    assert constructortag
    from cgen import Value

    gen = generator_factory(item_tags=("clazz", classtag, constructortag, "constructor_param"), counted=True, no_deco=True)
    return gen(Value(_type, name))


def template_parameter(classtag=None):
    assert classtag

    return generator_factory(item_tags=("clazz", classtag, "template_param"), counted=True)


def class_basename(classtag=None):
    assert classtag

    return generator_factory(item_tags=("clazz", classtag, "basename"))


def dump_accumulate_timer(name):
    gen = generator_factory(item_tags=("dump_timers"), no_deco=True)

    from dune.perftool.pdelab.localoperator import (name_time_dumper_os,
                                                    name_time_dumper_reset,
                                                    name_time_dumper_t,
                                                    name_time_dumper_counter)
    os = name_time_dumper_os()
    reset = name_time_dumper_reset()
    t = name_time_dumper_t()
    counter = name_time_dumper_counter()

    code = "DUMP_AND_ACCUMULATE_TIMER({},{},{},{},{});".format(name, os, reset, t, counter)
    return gen(code)
