from dune.perftool.generation import function_mangler
from loopy import CallMangleInfo
from loopy.symbolic import FunctionIdentifier
from loopy.types import NumpyType

import numpy


class LFSChild(FunctionIdentifier):
    def __init__(self, lfs):
        self.lfs = lfs

    def __getinitargs__(self):
        return (self.lfs,)

    @property
    def name(self):
        return '{}.child'.format(self.lfs)


@function_mangler
def lfs_child_mangler(target, func, dtypes):
    if isinstance(func, LFSChild):
        return CallMangleInfo(func.name, (NumpyType(str),), (NumpyType(numpy.int32),))


class CoefficientAccess(FunctionIdentifier):
    def __init__(self, container):
        self.container = container

    def __getinitargs__(self):
        return (self.container,)

    @property
    def name(self):
        return self.container


@function_mangler
def coefficient_mangler(target, func, dtypes):
    if isinstance(func, CoefficientAccess):
        return CallMangleInfo(func.name, (NumpyType(numpy.float64),), (NumpyType(str), NumpyType(numpy.int32)))


class PDELabAccumulationFunction(FunctionIdentifier):
    def __init__(self, accumobj, rank):
        self.accumobj = accumobj
        self.rank = rank

        assert rank in (1, 2)

    def __getinitargs__(self):
        return (self.accumobj, self.rank)


@function_mangler
def accumulation_mangler(target, func, dtypes):
    if isinstance(func, PDELabAccumulationFunction):
        if func.rank == 1:
            return CallMangleInfo('{}.accumulate'.format(func.accumobj),
                                  (),
                                  (NumpyType(str),
                                   NumpyType(numpy.int32),
                                   NumpyType(numpy.float64),
                                   )
                                  )
        if func.rank == 2:
            return CallMangleInfo('{}.accumulate'.format(func.accumobj),
                                  (),
                                  (NumpyType(str),
                                   NumpyType(numpy.int32),
                                   NumpyType(str),
                                   NumpyType(numpy.int32),
                                   NumpyType(numpy.float64),
                                   )
                                  )
